// XYZ Modules.scad
// Use these modules with the XYZ and IJK Coordinate Grids.

// include<XYZ Modules.scad>

//Braille Character Module
/*Adapted from Dr. Steven Sahyun's OpenSCAD 3D GS 8 Dot Label Generator, 
    which was adapted from OpenSCAD 3D Text Generator, Made by tdeagan, uploaded Apr 29, 2013
    http://www.thingiverse.com/make:37162
    and Braille OpenSCAD Font Module by drayde, published Nov 13, 2010
    http://www.thingiverse.com/thing:4758
*/
module drawtext(text) {
	//Characters - Table is incomplete
	gs8chars = ",;':-.!?()$&@/^_`{}~#%*+<=>×·÷0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyzαβγδεζηθικλμνξοπρστυφχψωΓΔΘΛΞΠΣΥΦΨΩ";

	/* Chracter table defining 2x4 GS8 braille characters
	Adapted from: http://www.geocities.com/dinceraydin/djlcdsim/chartable.js
    GS8 chars from https://en.wikipedia.org/wiki/Gardner–Salinas_braille_codes
    GS8 Braille dot cell placement
    [ 1, 2,
      3, 4,
      5, 6,
      7, 8 ]
*/
	gs8char_table = [ 
                   [ 0, 0, 1, 0, 0, 0, 0, 0], //,
                   [ 0, 0, 1, 0, 1, 0, 0, 0], //;
                   [ 0, 0, 0, 0, 1, 0, 0, 0], //'
                   [ 0, 0, 1, 1, 0, 0, 0, 0], //:
                   [ 0, 0, 0, 0, 1, 1, 0, 0], //-
                   [ 0, 0, 1, 1, 0, 1, 0, 0], //.
                   [ 0, 0, 1, 1, 1, 0, 0, 0], //!
                   [ 0, 0, 1, 1, 1, 1, 0, 0], //?
                   [ 0, 0, 1, 0, 1, 0, 0, 1], //(
                   [ 0, 1, 0, 1, 0, 0, 1, 0], //)
                   [ 0, 1, 0, 1, 1, 1, 1, 0], //$
                   [ 1, 1, 1, 0, 1, 1, 0, 0], //&
                   [ 0, 1, 0, 0, 1, 0, 0, 1], //@
                   [ 1, 0, 0, 0, 0, 1, 1, 1], // \ 
                   [ 0, 0, 0, 1, 0, 0, 1, 0], //^
                   [ 0, 0, 1, 0, 0, 0, 0, 1], //_
                   [ 0, 0, 0, 0, 1, 0, 1, 1], //`
                   [ 1, 0, 1, 1, 1, 1, 1, 1], //{
                   [ 0, 1, 1, 1, 1, 1, 1, 1], //}
                   [ 0, 0, 0, 0, 0, 1, 1, 1], //~
                   [ 0, 1, 0, 1, 1, 1, 0, 0], //#
                   [ 0, 1, 1, 1, 1, 0, 0, 0], //%
                   [ 0, 1, 0, 1, 1, 0, 0, 0], //*
                   [ 0, 0, 1, 1, 0, 0, 1, 0], //+    
                   //[ 0, 1, 0, 0, 1, 0, 1, 1], // /  can't put this in the chars string  
                   [ 0, 0, 0, 1, 1, 0, 0, 1], //<    
                   [ 0, 0, 1, 1, 1, 1, 1, 1], //=    
                   [ 0, 0, 1, 0, 0, 1, 1, 0], //>    
                   [ 0, 0, 1, 0, 0, 0, 1, 1], //×    
                   [ 0, 0, 1, 1, 0, 0, 0, 1], //·    
                   [ 0, 0, 0, 1, 0, 0, 1, 1], //÷    
                   [ 0, 1, 0, 0, 1, 1, 0, 0], //0
                   [ 1, 0, 0, 0, 0, 1, 0, 0], //1
                   [ 1, 0, 1, 0, 0, 1, 0, 0], //2
                   [ 1, 1, 0, 0, 0, 1, 0, 0], //3
                   [ 1, 1, 0, 1, 0, 1, 0, 0], //4
                   [ 1, 0, 0, 1, 0, 1, 0, 0], //5
                   [ 1, 1, 1, 0, 0, 1, 0, 0], //6
                   [ 1, 1, 1, 1, 0, 1, 0, 0], //7
                   [ 1, 0, 1, 1, 0, 1, 0, 0], //8
                   [ 0, 1, 1, 0, 0, 1, 0, 0], //9
                   [ 1, 0, 0, 0, 0, 0, 1, 0], //A
                   [ 1, 0, 1, 0, 0, 0, 1, 0], //B
                   [ 1, 1, 0, 0, 0, 0, 1, 0], //C
                   [ 1, 1, 0, 1, 0, 0, 1, 0], //D
                   [ 1, 0, 0, 1, 0, 0, 1, 0], //E
                   [ 1, 1, 1, 0, 0, 0, 1, 0], //F
                   [ 1, 1, 1, 1, 0, 0, 1, 0], //G
                   [ 1, 0, 1, 1, 0, 0, 1, 0], //H
                   [ 0, 1, 1, 0, 0, 0, 1, 0], //I
                   [ 0, 1, 1, 1, 0, 0, 1, 0], //J
                   [ 1, 0, 0, 0, 1, 0, 1, 0], //K
                   [ 1, 0, 1, 0, 1, 0, 1, 0], //L
                   [ 1, 1, 0, 0, 1, 0, 1, 0], //M
                   [ 1, 1, 0, 1, 1, 0, 1, 0], //N
                   [ 1, 0, 0, 1, 1, 0, 1, 0], //O
                   [ 1, 1, 1, 0, 1, 0, 1, 0], //P
                   [ 1, 1, 1, 1, 1, 0, 1, 0], //Q
                   [ 1, 0, 1, 1, 1, 0, 1, 0], //R
                   [ 0, 1, 1, 0, 1, 0, 1, 0], //S
                   [ 0, 1, 1, 1, 1, 0, 1, 0], //T
                   [ 1, 0, 0, 0, 1, 1, 1, 0], //U
                   [ 1, 0, 1, 0, 1, 1, 1, 0], //V
                   [ 0, 1, 1, 1, 0, 1, 1, 0], //W
                   [ 1, 1, 0, 0, 1, 1, 1, 0], //X
                   [ 1, 1, 0, 1, 1, 1, 1, 0], //Y
                   [ 1, 0, 0, 1, 1, 1, 1, 0], //Z
                   [ 1, 0, 0, 0, 0, 0, 0, 0], //a
                   [ 1, 0, 1, 0, 0, 0, 0, 0], //b
                   [ 1, 1, 0, 0, 0, 0, 0, 0], //c
                   [ 1, 1, 0, 1, 0, 0, 0, 0], //d
                   [ 1, 0, 0, 1, 0, 0, 0, 0], //e
                   [ 1, 1, 1, 0, 0, 0, 0, 0], //f
                   [ 1, 1, 1, 1, 0, 0, 0, 0], //g
                   [ 1, 0, 1, 1, 0, 0, 0, 0], //h
                   [ 0, 1, 1, 0, 0, 0, 0, 0], //i
                   [ 0, 1, 1, 1, 0, 0, 0, 0], //j
                   [ 1, 0, 0, 0, 1, 0, 0, 0], //k
                   [ 1, 0, 1, 0, 1, 0, 0, 0], //l
                   [ 1, 1, 0, 0, 1, 0, 0, 0], //m
                   [ 1, 1, 0, 1, 1, 0, 0, 0], //n
                   [ 1, 0, 0, 1, 1, 0, 0, 0], //o
                   [ 1, 1, 1, 0, 1, 0, 0, 0], //p
                   [ 1, 1, 1, 1, 1, 0, 0, 0], //q
                   [ 1, 0, 1, 1, 1, 0, 0, 0], //r
                   [ 0, 1, 1, 0, 1, 0, 0, 0], //s
                   [ 0, 1, 1, 1, 1, 0, 0, 0], //t
                   [ 1, 0, 0, 0, 1, 1, 0, 0], //u
                   [ 1, 0, 1, 0, 1, 1, 0, 0], //v
                   [ 0, 1, 1, 1, 0, 1, 0, 0], //w
                   [ 1, 1, 0, 0, 1, 1, 0, 0], //x
                   [ 1, 1, 0, 1, 1, 1, 0, 0], //y
                   [ 1, 0, 0, 1, 1, 1, 0, 0], //z  
                   [ 1, 0, 0, 0, 0, 0, 0, 1], //α
                   [ 1, 0, 1, 0, 0, 0, 0, 1], //β
                   [ 1, 1, 1, 1, 0, 0, 0, 1], //γ
                   [ 1, 1, 0, 1, 0, 0, 0, 1], //δ
                   [ 1, 0, 0, 1, 0, 0, 0, 1], //ε
                   [ 1, 0, 0, 1, 1, 1, 0, 1], //ζ
                   [ 1, 0, 0, 1, 0, 1, 0, 1], //η
                   [ 1, 1, 0, 1, 0, 1, 0, 1], //θ
                   [ 0, 1, 1, 0, 0, 0, 0, 1], //ι
                   [ 1, 0, 0, 0, 1, 0, 0, 1], //κ
                   [ 1, 0, 1, 0, 1, 0, 0, 1], //λ
                   [ 1, 1, 0, 0, 1, 0, 0, 1], //μ
                   [ 1, 1, 0, 1, 1, 0, 0, 1], //ν
                   [ 1, 1, 0, 0, 1, 1, 0, 1], //ξ
                   [ 1, 0, 0, 1, 1, 0, 0, 1], //ο
                   [ 1, 1, 1, 0, 1, 0, 0, 1], //π
                   [ 1, 0, 1, 1, 1, 0, 0, 1], //ρ
                   [ 0, 1, 1, 0, 1, 0, 0, 1], //σ
                   [ 0, 1, 1, 1, 1, 0, 0, 1], //τ
                   [ 1, 0, 0, 0, 1, 1, 0, 1], //υ
                   [ 1, 1, 1, 0, 0, 0, 0, 1], //φ
                   [ 1, 1, 1, 0, 1, 1, 0, 1], //χ
                   [ 1, 1, 0, 1, 1, 1, 0, 1], //ψ
                   [ 0, 1, 1, 1, 0, 1, 0, 1], //ω
                   [ 1, 1, 1, 1, 0, 0, 1, 1], //Γ
                   [ 1, 1, 0, 1, 0, 0, 1, 1], //Δ
                   [ 1, 1, 0, 1, 0, 1, 1, 1], //Θ
                   [ 1, 0, 1, 0, 1, 0, 1, 1], //Λ
                   [ 1, 1, 0, 0, 1, 1, 1, 1], //Ξ
                   [ 1, 1, 1, 0, 1, 0, 1, 1], //Π
                   [ 0, 1, 1, 0, 1, 0, 1, 1], //Σ
                   [ 1, 0, 0, 0, 1, 1, 1, 1], //Υ
                   [ 1, 1, 1, 0, 0, 0, 1, 1], //Φ
                   [ 1, 1, 0, 1, 1, 1, 1, 1], //Ψ
                   [ 0, 1, 1, 1, 0, 1, 1, 1] //Ω       
//                 [ 0, 0, 0, 0, 0, 0, 0, 0], //template for additional characters     
    ];
    
	//Process string one character at a time
	for(itext = [0:len(text)-1]) {
		//Convert character to index
        ichar = search(text[itext],gs8chars,1)[0];
        val = gs8char_table[ichar];
        //Decode character - rows each character has 4 rows and 2 columns
        //Select value to draw from table
        for(icol = [0:1]) {
            for(irow = [0:3]) {
                // Retrieve bit to draw
               translate([cellspacing*itext+dotspacing*icol, dotspacing*irow, 0])
               if(val[irow*2+icol]==1) { sphere(radius);}
            }
        }
	}
}
//Coordinate Arm Module
module coordinatearm(x, y, z) {
    cube([x, y, z], false);
}
//Triangle Module
module triangle (l, w, h){
   polyhedron(
               points=[ [0,0,0], //0
                       [l,0,0], //1
                       [l/2,w,0], //2
                       [0,0,h],  //3
                       [l,0,h], //4
                       [l/2,w,h] ], //5
    
               faces=[[0,1,2], 
                      [3,5,4], 
                      [0,2,5,3], //fixed
                      [0,3, 4, 1], //fixed
                      [1, 4, 5, 2]
    ]
               );}
//Label Module
module label(letter, size, height) {
    linear_extrude(height){
        text(letter, size, halign = "center", valign = "bottom");}
}